@@ -55,19 +55,23 @@ class FirewallCommand(object):
 
     def print_msg(self, msg=None):
         if msg is not None and not self.quiet:
-            print(msg)
+            sys.stdout.write(msg + "\n")
+
+    def print_error_msg(self, msg=None):
+        if msg is not None and not self.quiet:
+            sys.stderr.write(msg + "\n")
 
     def print_warning(self, msg=None):
         FAIL = '\033[91m'
         END = '\033[00m'
-        self.print_msg(FAIL + msg + END)
+        self.print_error_msg(FAIL + msg + END)
 
     def print_and_exit(self, msg=None, exit_code=0):
         #OK = '\033[92m'
         FAIL = '\033[91m'
         END = '\033[00m'
         if exit_code > 1:
-            self.print_msg(FAIL + msg + END)
+            self.print_error_msg(FAIL + msg + END)
         else:
             self.print_msg(msg)
             #self.print_msg(OK + msg + END)
@@ -77,66 +81,33 @@ class FirewallCommand(object):
         self.print_and_exit(msg, 2)
 
     def print_if_verbose(self, msg=None):
-        if msg and self.verbose:
-            print(msg)
+        if msg is not None and self.verbose:
+            sys.stdout.write(msg + "\n")
 
-    def __cmd_sequence(self, cmd_type, option, action_method, query_method,
-                       parse_method, message, start_args=None, end_args=None,
+    def __cmd_sequence(self, cmd_type, option, action_method, query_method, # pylint: disable=W0613, R0913, R0914
+                       parse_method, message, start_args=None, end_args=None, # pylint: disable=W0613
                        no_exit=False):
-        warn_type = {
-            "add": "ALREADY_ENABLED",
-            "remove": "NOT_ENABLED",
-        }
         if self.fw is not None:
             self.fw.authorizeAll()
         items = [ ]
         _errors = 0
+        _error_codes = [ ]
         for item in option:
             if parse_method is not None:
                 try:
                     item = parse_method(item)
                 except Exception as msg:
+                    code = FirewallError.get_code(str(msg))
                     if len(option) > 1:
                         self.print_warning("Warning: %s" % msg)
-                        continue
                     else:
-                        code = FirewallError.get_code(msg)
                         self.print_and_exit("Error: %s" % msg, code)
-
-            call_item = [ ]
-            if start_args is not None:
-                call_item += start_args
-            if not isinstance(item, list) and not isinstance(item, tuple):
-                call_item.append(item)
-            else:
-                call_item += item
-            self.deactivate_exception_handler()
-            try:
-                if cmd_type == "add" and not query_method(*call_item):
-                    items.append(item)
-                elif cmd_type == "remove" and query_method(*call_item):
-                    items.append(item)
-                else:
-                    if len(option) > 1:
-                        self.print_warning("Warning: %s: %s" % \
-                                           (warn_type[cmd_type],
-                                            message % item))
-                    else:
-                        code = FirewallError.get_code(warn_type[cmd_type])
-                        self.print_and_exit("Error: %s: %s" % \
-                                            (warn_type[cmd_type],
-                                             message % item), code)
+                    if code not in _error_codes:
+                        _error_codes.append(code)
                     _errors += 1
-            except DBusException as msg:
-                code = FirewallError.get_code(str(msg))
-                if len(option) > 1:
-                    self.print_warning("Warning: %s" % msg.get_dbus_message())
                     continue
-                else:
-                    self.print_and_exit("Error: %s" % msg.get_dbus_message(),
-                                        code)
-                _errors += 1
-            self.activate_exception_handler()
+
+            items.append(item)
 
         for item in items:
             call_item = [ ]
@@ -148,51 +119,77 @@ class FirewallCommand(object):
                 call_item += item
             if end_args is not None:
                 call_item += end_args
+            self.deactivate_exception_handler()
             try:
                 action_method(*call_item)
-            except DBusException as msg:
-                code = FirewallError.get_code(str(msg))
-                if len(option) > 1:
-                    self.print_warning("Warning: %s" % msg.get_dbus_message())
+            except (DBusException, Exception) as msg:
+                if isinstance(msg, DBusException):
+                    self.fail_if_not_authorized(msg.get_dbus_name())
+                    msg = msg.get_dbus_message()
                 else:
-                    self.print_and_exit("Error: %s" % msg.get_dbus_message(),
-                                        code)
+                    msg = str(msg)
+                code = FirewallError.get_code(msg)
+                if code in [ errors.ALREADY_ENABLED, errors.NOT_ENABLED,
+                             errors.ZONE_ALREADY_SET, errors.ALREADY_SET ]:
+                    code = 0
+                if len(option) > 1:
+                    self.print_warning("Warning: %s" % msg)
+                elif code == 0:
+                    self.print_warning("Warning: %s" % msg)
+                    return
+                else:
+                    self.print_and_exit("Error: %s" % msg, code)
+                if code not in _error_codes:
+                    _error_codes.append(code)
                 _errors += 1
+            self.activate_exception_handler()
 
-        if _errors == len(option) and not no_exit:
-            sys.exit(0)
+        if not no_exit:
+            if len(option) > _errors or 0 in _error_codes:
+                # There have been more options than errors or there
+                # was at least one error code 0, return.
+                return
+            elif len(_error_codes) == 1:
+                # Exactly one error code, use it.
+                sys.exit(_error_codes[0])
+            elif len(_error_codes) > 1:
+                # There is more than error, exit using
+                # UNKNOWN_ERROR. This could happen within sequences
+                # where parsing failed with different errors like
+                # INVALID_PORT and INVALID_PROTOCOL.
+                sys.exit(errors.UNKNOWN_ERROR)
 
-    def add_sequence(self, option, action_method, query_method, parse_method,
+    def add_sequence(self, option, action_method, query_method, parse_method, # pylint: disable=R0913
                      message, no_exit=False):
         self.__cmd_sequence("add", option, action_method, query_method,
                             parse_method, message, no_exit=no_exit)
 
-    def x_add_sequence(self, x, option, action_method, query_method,
+    def x_add_sequence(self, x, option, action_method, query_method, # pylint: disable=R0913
                        parse_method, message, no_exit=False):
         self.__cmd_sequence("add", option, action_method, query_method,
                             parse_method, message, start_args=[x],
                             no_exit=no_exit)
 
-    def zone_add_timeout_sequence(self, zone, option, action_method,
+    def zone_add_timeout_sequence(self, zone, option, action_method, # pylint: disable=R0913
                                   query_method, parse_method, message,
                                   timeout, no_exit=False):
         self.__cmd_sequence("add", option, action_method, query_method,
                             parse_method, message, start_args=[zone],
                             end_args=[timeout], no_exit=no_exit)
 
-    def remove_sequence(self, option, action_method, query_method,
+    def remove_sequence(self, option, action_method, query_method, # pylint: disable=R0913
                         parse_method, message, no_exit=False):
         self.__cmd_sequence("remove", option, action_method, query_method,
                             parse_method, message, no_exit=no_exit)
 
-    def x_remove_sequence(self, x, option, action_method, query_method,
+    def x_remove_sequence(self, x, option, action_method, query_method, # pylint: disable=R0913
                           parse_method, message, no_exit=False):
         self.__cmd_sequence("remove", option, action_method, query_method,
                             parse_method, message, start_args=[x],
                             no_exit=no_exit)
 
 
-    def __query_sequence(self, option, query_method, parse_method, message,
+    def __query_sequence(self, option, query_method, parse_method, message, # pylint: disable=R0913
                          start_args=None, no_exit=False):
         items = [ ]
         for item in option:
@@ -204,7 +201,7 @@ class FirewallCommand(object):
                         self.print_warning("Warning: %s" % msg)
                         continue
                     else:
-                        code = FirewallError.get_code(msg)
+                        code = FirewallError.get_code(str(msg))
                         self.print_and_exit("Error: %s" % msg, code)
             items.append(item)
 
@@ -220,13 +217,20 @@ class FirewallCommand(object):
             try:
                 res = query_method(*call_item)
             except DBusException as msg:
-                code = FirewallError.get_code(str(msg))
+                self.fail_if_not_authorized(msg.get_dbus_name())
+                code = FirewallError.get_code(msg.get_dbus_message())
                 if len(option) > 1:
                     self.print_warning("Warning: %s" % msg.get_dbus_message())
                     continue
                 else:
                     self.print_and_exit("Error: %s" % msg.get_dbus_message(),
                                         code)
+            except Exception as msg:
+                code = FirewallError.get_code(str(msg))
+                if len(option) > 1:
+                    self.print_warning("Warning: %s" % msg)
+                else:
+                    self.print_and_exit("Error: %s" % msg, code)
             self.activate_exception_handler()
             if len(option) > 1:
                 self.print_msg("%s: %s" % (message % item, ("no", "yes")[res]))
@@ -235,12 +239,12 @@ class FirewallCommand(object):
         if not no_exit:
             sys.exit(0)
 
-    def query_sequence(self, option, query_method, parse_method, message,
+    def query_sequence(self, option, query_method, parse_method, message, # pylint: disable=R0913
                        no_exit=False):
         self.__query_sequence(option, query_method, parse_method,
                               message, no_exit=no_exit)
 
-    def x_query_sequence(self, x, option, query_method, parse_method,
+    def x_query_sequence(self, x, option, query_method, parse_method, # pylint: disable=R0913
                          message, no_exit=False):
         self.__query_sequence(option, query_method, parse_method,
                               message, start_args=[x], no_exit=no_exit)
@@ -263,9 +267,10 @@ class FirewallCommand(object):
                                 "portid[-portid]%sprotocol" % separator)
         if not check_port(port):
             raise FirewallError(errors.INVALID_PORT, port)
-        if proto not in [ "tcp", "udp" ]:
+        if proto not in [ "tcp", "udp", "sctp", "dccp" ]:
             raise FirewallError(errors.INVALID_PROTOCOL,
-                                "'%s' not in {'tcp'|'udp'}" % proto)
+                                "'%s' not in {'tcp'|'udp'|'sctp'|'dccp'}" % \
+                                proto)
         return (port, proto)
 
     def parse_forward_port(self, value):
@@ -297,9 +302,10 @@ class FirewallCommand(object):
 
         if not check_port(port):
             raise FirewallError(errors.INVALID_PORT, port)
-        if protocol not in [ "tcp", "udp" ]:
+        if protocol not in [ "tcp", "udp", "sctp", "dccp" ]:
             raise FirewallError(errors.INVALID_PROTOCOL,
-                                "'%s' not in {'tcp'|'udp'}" % protocol)
+                                "'%s' not in {'tcp'|'udp'|'sctp'|'dccp'}" % \
+                                protocol)
         if toport and not check_port(toport):
             raise FirewallError(errors.INVALID_PORT, toport)
         if toaddr and not check_single_address("ipv4", toaddr):
@@ -341,7 +347,25 @@ class FirewallCommand(object):
                                 (value, "', '".join(ipvs)))
         return value
 
-    def print_zone_info(self, zone, settings, default_zone=None):
+    def check_helper_family(self, value):
+        ipvs = [ "", "ipv4", "ipv6" ]
+        if value not in ipvs:
+            raise FirewallError(errors.INVALID_IPV,
+                                "invalid argument: %s (choose from '%s')" % \
+                                (value, "', '".join(ipvs)))
+        return value
+
+    def check_module(self, value):
+        if not value.startswith("nf_conntrack_"):
+            raise FirewallError(
+                errors.INVALID_MODULE,
+                "'%s' does not start with 'nf_conntrack_'" % value)
+        if len(value.replace("nf_conntrack_", "")) < 1:
+            raise FirewallError(errors.INVALID_MODULE,
+                                "Module name '%s' too short" % value)
+        return value
+
+    def print_zone_info(self, zone, settings, default_zone=None): # pylint: disable=R0914
         target = settings.getTarget()
         icmp_block_inversion = settings.getIcmpBlockInversion()
         interfaces = settings.getInterfaces()
@@ -384,7 +408,7 @@ class FirewallCommand(object):
                                     (port, proto, toport, toaddr)
                                     for (port, proto, toport, toaddr) in \
                                     forward_ports]))
-        self.print_msg("  sourceports: " +
+        self.print_msg("  source-ports: " +
                        " ".join(["%s/%s" % (port[0], port[1])
                                  for port in source_ports]))
         self.print_msg("  icmp-blocks: " + " ".join(icmp_blocks))
@@ -440,6 +464,21 @@ class FirewallCommand(object):
                                                  for k, v in options.items()]))
         self.print_msg("  entries: " + " ".join(entries))
 
+    def print_helper_info(self, helper, settings):
+        ports = settings.getPorts()
+        module = settings.getModule()
+        family = settings.getFamily()
+        description = settings.getDescription()
+        short_description = settings.getShort()
+        self.print_msg(helper)
+        if self.verbose:
+            self.print_msg("  summary: " + short_description)
+            self.print_msg("  description: " + description)
+        self.print_msg("  family: " + family)
+        self.print_msg("  module: " + module)
+        self.print_msg("  ports: " + " ".join(["%s/%s" % (port[0], port[1])
+                                               for port in ports]))
+
     def print_query_result(self, value):
         if value:
             self.print_and_exit("yes")
@@ -449,17 +488,19 @@ class FirewallCommand(object):
     def exception_handler(self, exception_message):
         if not self.__use_exception_handler:
             raise
+        self.fail_if_not_authorized(exception_message)
+        code = FirewallError.get_code(str(exception_message))
+        if code in [ errors.ALREADY_ENABLED, errors.NOT_ENABLED,
+                     errors.ZONE_ALREADY_SET, errors.ALREADY_SET ]:
+            self.print_warning("Warning: %s" % exception_message)
+        else:
+            self.print_and_exit("Error: %s" % exception_message, code)
+
+    def fail_if_not_authorized(self, exception_message):
         if "NotAuthorizedException" in exception_message:
             msg = """Authorization failed.
     Make sure polkit agent is running or run the application as superuser."""
             self.print_and_exit(msg, errors.NOT_AUTHORIZED)
-        else:
-            code = FirewallError.get_code(exception_message)
-            if code in [ errors.ALREADY_ENABLED, errors.NOT_ENABLED,
-                         errors.ZONE_ALREADY_SET ]:
-                self.print_warning("Warning: %s" % exception_message)
-            else:
-                self.print_and_exit("Error: %s" % exception_message, code)
 
     def deactivate_exception_handler(self):
         self.__use_exception_handler = False
@@ -469,6 +510,7 @@ class FirewallCommand(object):
 
     def get_ipset_entries_from_file(self, filename):
         entries = [ ]
+        entries_set = set()
         f = open(filename)
         for line in f:
             if not line:
@@ -476,7 +518,8 @@ class FirewallCommand(object):
             line = line.strip()
             if len(line) < 1 or line[0] in ['#', ';']:
                 continue
-            if line not in entries:
+            if line not in entries_set:
                 entries.append(line)
+                entries_set.add(line)
         f.close()
         return entries
