@@ -114,6 +114,7 @@ class Firewall(object):
         self._automatic_helpers = config.FALLBACK_AUTOMATIC_HELPERS
         self.nf_conntrack_helper_setting = 0
         self.nf_conntrack_helpers = { }
+        self.nf_nat_helpers = { }
 
     def individual_calls(self):
         return self._individual_calls
@@ -203,8 +204,18 @@ class Firewall(object):
                     log.debug1("  %s: %s", key, ", ".join(values))
             else:
                 log.debug1("No conntrack helpers supported by the kernel.")
+
+            self.nf_nat_helpers = functions.get_nf_nat_helpers()
+            if len(self.nf_nat_helpers) > 0:
+                log.debug1("NAT helpers supported by the kernel:")
+                for key,values in self.nf_nat_helpers.items():
+                    log.debug1("  %s: %s", key, ", ".join(values))
+            else:
+                log.debug1("No NAT helpers supported by the kernel.")
+
         else:
             self.nf_conntrack_helpers = { }
+            self.nf_nat_helpers = { }
             log.warning("modinfo command is missing, not able to detect conntrack helpers.")
 
     def _start(self, reload=False, complete_reload=False):
@@ -989,9 +1000,10 @@ class Firewall(object):
     def check_tcpudp(self, protocol):
         if not protocol:
             raise FirewallError(errors.MISSING_PROTOCOL)
-        if protocol not in [ "tcp", "udp" ]:
+        if protocol not in [ "tcp", "udp", "sctp", "dccp" ]:
             raise FirewallError(errors.INVALID_PROTOCOL,
-                                "'%s' not in {'tcp'|'udp'}" % protocol)
+                                "'%s' not in {'tcp'|'udp'|'sctp'|'dccp'}" % \
+                                protocol)
 
     def check_ip(self, ip):
         if not functions.checkIP(ip):
